using System;
using Dropit.Extension.Core;
using EPiServer.Core;
using EPiServer.DynamicContent;
using EPiServer.PlugIn;
using DV.Properties;
using System.Text;
using System.Text.RegularExpressions;
using EPiServer.Core.Html;
using EPiServer.Filters;

namespace EPiServer.Composer.Functions
{
    /// <summary>
    /// Listing Block for Composer.
    /// </summary>
    public partial class CategoryListing : BaseContentFunction
    {
        private const string STARTPOINT = "StartPoint";
        private const string CATEGORY = "Category";
        private const string START = "Start";
        private const string END = "End";
        private const string SORTING = "Sorting";
        private const string SHOWDATE = "ShowDate";
        private const string SHOWPREVIEWTEXT = "ShowPreviewText";

        protected PageReference StartPoint { get; set; }
        protected PropertySelectOrAddCategory Category { get; set; }
        protected PropertyDate Start { get; set; }
        protected PropertyDate End { get; set; }
        protected SortingProperty Sorting { get; set; }
        //protected PropertyBoolean ShowDate { get; set; }
        //protected PropertyBoolean ShowPreviewText { get; set; }

        /// <summary>
        /// Load the List from ListRoot
        /// </summary>
        /// <param name="e"></param>
        protected override void OnLoad(EventArgs e)
        {
            Category = (PropertySelectOrAddCategory)ContentFunctionData.Property[CATEGORY];

            // If there are no categories, there's nothing to do here.  Don't continue
            if(Category.Category.Count > 0)
            {
                Sorting = (SortingProperty)ContentFunctionData.Property[SORTING];      
                Start = (PropertyDate)ContentFunctionData.Property[START];
                End = (PropertyDate)ContentFunctionData.Property[END];            

                if (null != ContentFunctionData.Property[STARTPOINT])
                {
                    StartPoint = ((PropertyPageReference)ContentFunctionData.Property[STARTPOINT]).PageLink;
                }

                // Default to the config'd StartPage if nothing else is set
                if (null == StartPoint)
                {
                    StartPoint = PageReference.StartPage;
                }

                PropertyCriteriaCollection pcc = new PropertyCriteriaCollection();

                if (Start.Value != null)
                {
                    PropertyCriteria pc1 = new PropertyCriteria()
                    {
                        Name = "PageStartPublish",
                        Condition = CompareCondition.GreaterThan,
                        Required = true,
                        Type = PropertyDataType.Date,
                        Value = ((DateTime)Start.Value).ToString()
                    };
                    pcc.Add(pc1);
                }
                else if (End.Value != null)
                {
                    PropertyCriteria pc2 = new PropertyCriteria()
                    {
                        Name = "PageStartPublish",
                        Condition = CompareCondition.LessThan,
                        Required = true,
                        Type = PropertyDataType.Date,
                        Value = ((DateTime)End.Value).ToString()
                    };
                    pcc.Add(pc2);
                }
                else
                {
                    //if (Category != null) // already checked this above
                    {
                        foreach (var item in Category.CategoryNames)
                        {
                            PropertyCriteria pc1 = new PropertyCriteria()
                            {
                                Name = "PageCategory",
                                Condition = Filters.CompareCondition.Equal,
                                Type = PropertyDataType.Category,
                                Value = item.ToString(),
                                Required = false
                            };
                            pcc.Add(pc1);
                        }
                    }
                }

                var ds = DataFactory.Instance.FindPagesWithCriteria(StartPoint, pcc);

                if (null != ds)
                {
                    if (Start != null && Category != null)
                    {
                        new FilterAtLeastOneCategory(Category.Category).Filter(ds);
                    }

                    FilterForVisitor.Filter(ds);

                    if (Sorting != null)
                    {
                        if (Sorting.Value as string == SortOrder.Alphabetical.ToString())
                        {
                            PageList.SortOrder = FilterSortOrder.Alphabetical;
                        }
                        else if (Sorting.Value as string == SortOrder.ChangeDateNew.ToString())
                        {
                            PageList.SortOrder = FilterSortOrder.ChangedDescending;
                        }
                        else if (Sorting.Value as string == SortOrder.CreateDateNew.ToString())
                        {
                            PageList.SortOrder = FilterSortOrder.CreatedDescending;
                        }
                        else if (Sorting.Value as string == SortOrder.CreateDateOld.ToString())
                        {
                            PageList.SortOrder = FilterSortOrder.CreatedAscending;
                        }
                        else if (Sorting.Value as string == SortOrder.PublishDateNew.ToString())
                        {
                            PageList.SortOrder = FilterSortOrder.PublishedDescending;
                        }
                        else if (Sorting.Value as string == SortOrder.PublishDateOld.ToString())
                        {
                            PageList.SortOrder = FilterSortOrder.PublishedAscending;
                        }
                    }

                    PageList.DataSource = ds;
                    PageList.DataBind();
                }
            }

            base.OnLoad(e);                    
        }
        
        public string GetPublishedText(PageData page)
        {
            if (ContentFunctionData.Property[SHOWDATE].Value != null)
            {
                return page.StartPublish.ToString();
            }
            return String.Empty;
        }

        private string StripPreviewText(string previewText, int maxLength)
        {
            if (previewText.Length <= maxLength)
            {
                return previewText;
            }
            previewText = previewText.Substring(0, maxLength);

            // The maximum number of characters to cut from the end of the string.
            int maxCharCut = (previewText.Length > 15 ? 15 : previewText.Length - 1);
            int previousWord = previewText.LastIndexOfAny(new char[] { ' ', '.', ',', '!', '?' }, previewText.Length - 1, maxCharCut);
            if (previousWord <= 0)
            {
                previewText = previewText.Substring(0, previousWord);
            }
            return previewText + " ...";
        }

        public string GetPreviewText(PageData page)
        {            
            if (ContentFunctionData.Property[SHOWPREVIEWTEXT].Value != null)
            {
                int previewTextLength = 100;

                if (page.Property["MainIntro"] != null && page.Property["MainIntro"].ToString().Length > 0)
                {
                    return StripPreviewText(page.Property["MainIntro"].ToWebString(), previewTextLength);
                }

                string previewText = String.Empty;

                if (page.Property["MainBody"] != null)
                {
                    previewText = page.Property["MainBody"].ToWebString();
                }

                if (String.IsNullOrEmpty(previewText))
                {
                    return string.Empty;
                }

                //If the MainBody contains DynamicContents, replace those with an empty string
                StringBuilder regexPattern = new StringBuilder(@"<span[\s\W\w]*?classid=""");
                regexPattern.Append(DynamicContentFactory.Instance.DynamicContentId.ToString());
                regexPattern.Append(@"""[\s\W\w]*?</span>");
                previewText = Regex.Replace(previewText, regexPattern.ToString(), string.Empty, RegexOptions.IgnoreCase | RegexOptions.Multiline);

                return TextIndexer.StripHtml(previewText, previewTextLength);
            }
            else return String.Empty;
        }
    }

    public class FilterAtLeastOneCategory : IPageFilter
    {
        public CategoryList Categories
        {
            get;
            set;
        }

        public FilterAtLeastOneCategory()
        {
        }

        public FilterAtLeastOneCategory(CategoryList categoryList)
        {
            this.Categories = categoryList;
        }

        public void Filter(PageDataCollection pages)
        {
            for (int i = pages.Count - 1; i >= 0; i--)
            {
                bool isMatch = false;
                var categoryList = pages[i].Category;

                if (categoryList != null)
                {
                    foreach (var category in categoryList)
                    {
                        foreach (var currentCategory in this.Categories)
                        {
                            if (category == currentCategory)
                            {
                                isMatch = true;
                            }
                        }
                    }
                }

                if (!isMatch)
                {
                    pages.RemoveAt(i);
                }
            }
        }

        public void Filter(object sender, FilterEventArgs e)
        {
            this.Filter(e.Pages);
        }

        public bool ShouldFilter(PageData page)
        {
            throw new NotImplementedException();
        }
    }
}